/* Copyright 2011-2012 Profict Holding 
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package nl.trivento.albero.parsers.groovy

import nl.trivento.albero.configuration.Configuration
import nl.trivento.albero.model.CompositeTree
import nl.trivento.albero.model.Tree
import nl.trivento.albero.parsers.Parser
import nl.trivento.albero.parsers.groovy.builders.TreeContainer

/**
 * Parses <a href="http://groovy.codehaus.org">Groovy</a> scripts.
 *
 */
class GroovyParser implements Parser {
	private Configuration configuration

	private GroovyClassLoader loader

	private Map cache = [:]

	/**
	 * Creates a Groovy parser.
	 */
	public GroovyParser() {}

	String getName() {
		"groovy"
	}

	void initialise(Configuration configuration, Map<String, String> parameters) {
		this.configuration = configuration

		loader = new GroovyClassLoader()
	}

	Tree parse(String tree) {
		cache.containsKey(tree) ? cache[tree] : parseAndCache(tree)
	}

	private Tree parseAndCache(String tree) {
		def treeContainer = new TreeContainer(configuration)

		def treeConfigurer = loader.parseClass(tree).newInstance()
		treeConfigurer.metaClass.methodMissing = {String name, args ->
			treeContainer."$name"(*args)
		}
		treeConfigurer.run()

		Tree parsedTree = treeContainer.build()

		if (!(parsedTree instanceof CompositeTree)) {
			cache[tree] = parsedTree
		}

		parsedTree
	}

	void destroy() {}
}
